/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * include/dt-bindings/media/omap3-isp.h
 *
 * Copyright (C) 2015 Sakari Ailus
 */

#ifndef __DT_BINDINGS_OMAP3_ISP_H__
#define __DT_BINDINGS_OMAP3_ISP_H__

#define OMAP3ISP_PHY_TYPE_COMPLEX_IO	0
#define OMAP3ISP_PHY_TYPE_CSIPHY	1

#endif /* __DT_BINDINGS_OMAP3_ISP_H__ */
