/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __NET_PSP_ALL_H
#define __NET_PSP_ALL_H

#include <uapi/linux/psp.h>
#include <net/psp/functions.h>
#include <net/psp/types.h>

/* Do not add any code here. Put it in the sub-headers instead. */

#endif /* __NET_PSP_ALL_H */
