/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef __DT_BINDINGS_RESET_SAMA7G5_H
#define __DT_BINDINGS_RESET_SAMA7G5_H

#define SAMA7G5_RESET_USB_PHY1		4
#define SAMA7G5_RESET_USB_PHY2		5
#define SAMA7G5_RESET_USB_PHY3		6

#endif /* __DT_BINDINGS_RESET_SAMA7G5_H */
