/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (c) 2019, The Linux Foundation. All rights reserved.
 * Copyright (c) 2024, Linaro Limited
 */

#ifndef _DT_BINDINGS_RESET_QCOM_GPU_CC_SAR2130P_H
#define _DT_BINDINGS_RESET_QCOM_GPU_CC_SAR2130P_H

#define GPUCC_GPU_CC_GX_BCR			0
#define GPUCC_GPU_CC_ACD_BCR			1
#define GPUCC_GPU_CC_GX_ACD_IROOT_BCR		2

#endif
