/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Common values for the Polyval hash algorithm
 *
 * Copyright 2021 Google LLC
 */

#ifndef _CRYPTO_POLYVAL_H
#define _CRYPTO_POLYVAL_H

#define POLYVAL_BLOCK_SIZE	16
#define POLYVAL_DIGEST_SIZE	16

#endif
